/* fgctrl.c: fgctrl board controller

FlighGear Controller, a simplfied cockpit for FlightGear and other simulators.
Copyright (C) 2012 Nachanon Vetjasit, Bulakorn Suwanmalai

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

A copy of GNU General Public License is included in the
FlightGear Controller distribution in the file COPYING.txt.
If you did not recieve this copy, write to the Free Software Foundation, Inc.,
51 Franklin St, 17 Fifth Floor, Boston, MA 02110-1301, USA. */

#include <avr/io.h>
#include "fgctrl.h"

/* _read_adc(): read analog port */
static uint16_t _read_adc(uint8_t channel) {
	ADMUX = 0b01000000;
	ADMUX |= channel;
	ADCSRA = 0b11000110;
	
	while ((ADCSRA & (1<<ADSC)));
	return ADCL + ADCH*256;
}

/* _fgctrlmode(): query fgctrl mode switch (0: off, 1:on) */
static uint8_t _fgctrlmode() {
	return ((PINC & (1<<(4)))!=0);
}

/* _fgctrlaxisinvert(): invert the fgctrl axis value */
static uint16_t _fgctrlaxisinvert(uint16_t val) {
	return 1023-val;
}

/* _fgctrlaxis_y_avg(): average/delay fgctrl y axis value */
static uint16_t _fgctrlaxis_y_avg(uint16_t val) {
	static uint16_t queue[10];	
	static uint8_t num=0; 
	uint8_t i;
	uint16_t avg=0;
	
	if(num < 10) {
		num++;
	} else {
		for(i=1;i<10;i++) queue[i-1] = queue[i];
	}
	
	queue[num-1] = val;
	
	for(i=0;i<num;i++)
		avg += queue[i];
	avg = avg/num;
	
	return avg;
}

/* fgctrlaxis(): query fgctrl axis status (range 0 to 1023) */
uint16_t fgctrlaxis(uint8_t ch) {
	uint16_t val;
	
	if(ch >= 4) return 0;
	
	val=_read_adc(ch);
	if(ch==1) val=_fgctrlaxis_y_avg(val);
	if(ch==2) val=_fgctrlaxisinvert(val);
	
	return val;
}

/* fgctrlbutton(): query fgctrl button status (range 0 to 1) */
uint8_t fgctrlbutton(uint8_t ch) {
	uint8_t val;
	
	if(ch <=5)
		val=((PINB & (1<<(ch)))!=0);
	else if(ch == 6 || ch == 7) 
		val=((PIND & (1<<(ch-6)))!=0);
	else if(ch == 8 || ch == 9)
		val=((PIND & (1<<(ch-3)))!=0);
	else if(ch == 10)
		val=((PINC & (1<<(ch-5)))!=0);
	else return 0;
	
	if(!_fgctrlmode()) val=!val;
	return val;
}

/* fgctrlinit(): prepare pins and registers for fgctrl board */
void fgctrlinit() {
	/* set all Port C to input state */
	DDRC=0x00;
	/* disable all Port C pullups */
	PORTC=0x00;
	
	/* set all Port B to input state */ 
	PORTB=0x00;
	/* disable all Port B pullups */
	DDRB=0x00;
	
	/* set usable bit of Port D to input state */
	DDRD=0b10011100 & DDRD;
	/* disable usable bit of Port D pullups */
	PORTD=0b10011100 & PORTD;
}

