/* main.c: main fgctrl-avr program, usb handler

FlighGear Controller, a simplfied cockpit for FlightGear and other simulators.
Copyright (C) 2012 Nachanon Vetjasit, Bulakorn Suwanmalai

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

A copy of GNU General Public License is included in the
FlightGear Controller distribution in the file COPYING.txt.
If you did not recieve this copy, write to the Free Software Foundation, Inc.,
51 Franklin St, 17 Fifth Floor, Boston, MA 02110-1301, USA. */

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include <avr/delay.h>
#include <string.h>

#include "usbdrv.h"
#include "fgctrl.h"

static uchar    idleRate=2;

static struct ReportBuffer
{
   int8_t x;
   int8_t y;
   int8_t z;
   int8_t zc;
   char b0:1;
   char b1:1;
   char b2:1;
   char b3:1;
   char b4:1;
   char b5:1;
   char b6:1;
   char b7:1;
   char b8:1;
   char b9:1;            
   char b10:1;
   char unused:5;
} reportBuffer;

static void fgctrlBuildReport(struct ReportBuffer *reportBuffer);

uchar usbFunctionSetup(uchar data[8])
{
	usbRequest_t    *rq = (void *)data;

	usbMsgPtr = (uchar*)&reportBuffer;
	if((rq->bmRequestType & USBRQ_TYPE_MASK) == USBRQ_TYPE_CLASS) {
		/* class request type */
		
		if(rq->bRequest == USBRQ_HID_GET_REPORT){
			/* wValue: ReportType (highbyte), ReportID (lowbyte) */
			/* we only have one report type, so don't look at wValue */
			
			fgctrlBuildReport(&reportBuffer);
			usbMsgPtr = (uchar*)&reportBuffer;
			return sizeof(reportBuffer);
			
		}else if(rq->bRequest == USBRQ_HID_GET_IDLE){
			usbMsgPtr = &idleRate;
			return 1;
		}else if(rq->bRequest == USBRQ_HID_SET_IDLE){
			idleRate = rq->wValue.bytes[1];
		}
	} else {
		/* no vendor specific requests implemented */
	}
	return 0;
}


static void fgctrlBuildReport(struct ReportBuffer *reportBuffer)
{
	if (reportBuffer != NULL)
	{
		reportBuffer->x   = (int8_t)( (((int16_t)fgctrlaxis(0))-512) /4);
		reportBuffer->y   = (int8_t)( (((int16_t)fgctrlaxis(1))-512) /4);
		reportBuffer->z   = (int8_t)( (((int16_t)fgctrlaxis(2))-512) /4);
		reportBuffer->zc  = (int8_t)( (((int16_t)fgctrlaxis(3))-512) /4);
		reportBuffer->b0  = fgctrlbutton(0);
		reportBuffer->b1  = fgctrlbutton(1);
		reportBuffer->b2  = fgctrlbutton(2);
		reportBuffer->b3  = fgctrlbutton(3);
		reportBuffer->b4  = fgctrlbutton(4);
		reportBuffer->b5  = fgctrlbutton(5);
		reportBuffer->b6  = fgctrlbutton(6);
		reportBuffer->b7  = fgctrlbutton(7);
		reportBuffer->b8  = fgctrlbutton(8);
		reportBuffer->b9  = fgctrlbutton(9);
		reportBuffer->b10 = fgctrlbutton(10);
	}
}

char usbDescriptorHidReport[52] PROGMEM = {
    0x05, 0x01,                    // USAGE_PAGE (Generic Desktop)
    0x09, 0x05,                    // USAGE (Game Pad)
    0xa1, 0x01,                    // COLLECTION (Application)
    0x09, 0x01,                    //   USAGE (Pointer)
    0xa1, 0x00,                    //   COLLECTION (Physical)
    0x09, 0x30,                    //     USAGE (X)
    0x09, 0x31,                    //     USAGE (Y)
    0x09, 0x32,                    //     USAGE (Z)
    0x09, 0x33,                    //     USAGE (ZCr)
    0x15, 0x80,                    //   LOGICAL_MINIMUM (-128)
    0x25, 0x7F,                    //     LOGICAL_MAXIMUM (127)
    0x75, 0x08,                    //   REPORT_SIZE (8)
    0x95, 0x04,                    //   REPORT_COUNT (4)
    0x81, 0x02,                    //   INPUT (Data,Var,Abs)
    0xc0,                          // END_COLLECTION
    0x05, 0x09,                    // USAGE_PAGE (Button)
    0x19, 0x01,                    //   USAGE_MINIMUM (Button 1)
    0x29, 0x0B,                    //   USAGE_MAXIMUM (Button 11)
    0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
    0x25, 0x01,                    //   LOGICAL_MAXIMUM (1)
    0x75, 0x01,                    // REPORT_SIZE (1)
    0x95, 0x0B,                    // REPORT_COUNT (11)
    0x81, 0x02,                    // INPUT (Data,Var,Abs)
    /* filler- to complete the byte */
    0x95, 0x01,                    //     REPORT_COUNT (1)
    0x75, 0x05,                    //     REPORT_SIZE (5)
    0x81, 0x03,                    //     INPUT (Cnst,Var,Abs)
    /* end filler */
    0xc0                           // END_COLLECTION
};



int main(void)
{
    fgctrlinit();
	
	_delay_ms(10); 
	
	usbInit();
	
	usbDeviceDisconnect();
	_delay_ms(300);
	usbDeviceConnect();
	
	sei();
	
	/*DDRD|=0b00001000;
	
	PORTD|=0b00001000;*/
	
	for(;;){	/* main event loop */
		usbPoll();
		
		/* is host waiting for us to report the position? */
		if(usbInterruptIsReady()){
			fgctrlBuildReport(&reportBuffer);
			usbSetInterrupt((uchar*)&reportBuffer, sizeof(reportBuffer));
		}
	}
	return 0;
}
